<?php
/** @noinspection PhpMissingFieldTypeInspection */
declare(strict_types=1);

namespace Inti\Provider\Service;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Inti\Provider\ProviderInterface;
use Laminas\ServiceManager\AbstractPluginManager;

use function class_exists;
use function interface_exists;

/**
 * Class ProviderManager
 *
 * @package Inti\Provider\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-provider
 *
 * @method ProviderInterface|object get(string $id)
 */
final class ProviderManager extends AbstractPluginManager implements ProviderManagerInterface
{
    protected $instanceOf = ProviderInterface::class;

    private Collection $providers;

    /**
     * @param string $type
     * @param string $providedType
     * @return bool
     */
    public function canProvide(string $type, string $providedType): bool
    {
        return $this->getProvider($type, $providedType) instanceof ProviderInterface;
    }

    /**
     * @param string $type
     * @param string $providedType
     * @return ProviderInterface|null
     */
    public function getProvider(string $type, string $providedType): ?ProviderInterface
    {
        return $this->getProviders($type, $providedType)[0] ?? null;
    }

    /**
     * @param string|null $type
     * @param string|null $providedType
     * @return ProviderInterface[]
     */
    public function getProviders(?string $type = null, ?string $providedType = null): array
    {
        return $this->getAllRegisteredProviders()->filter(function (ProviderInterface $provider) use ($type, $providedType) {
            if (empty($type) && empty($providedType)) {
                return true;
            }

            if (!empty($type) && !$provider instanceof $type) {
                return false;
            }

            if (!empty($providedType)) {
                $isObjectType = class_exists($providedType) || interface_exists($providedType);

                if ($isObjectType && $provider->getProvidedType() instanceof $providedType) {
                    return true;
                }

                if ($provider->getProvidedType()->getName() === $providedType) {
                    return true;
                }

                return false;
            }

            return true;
        })->getValues();
    }

    /**
     * @return Collection|ProviderInterface[]
     */
    private function getAllRegisteredProviders(): Collection
    {
        if (!isset($this->providers)) {
            $this->providers = new ArrayCollection();

            $lists = [
                $this->services,
                $this->factories,
            ];

            foreach ($lists as $list) {
                foreach ($list as $id => $value) {
                    $provider = $this->get($id);

                    if (!$this->providers->contains($provider)) {
                        $this->providers->add($provider);
                    }
                }
            }
        }

        return $this->providers;
    }
}
